@@ -5,7 +5,7 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# Author: Kyle Lahnakoski (kyle@lahnakoski.com)
+# Contact: Kyle Lahnakoski (kyle@lahnakoski.com)
 #
 from __future__ import absolute_import, division, unicode_literals
 
@@ -24,7 +24,6 @@ _Query = None
 
 
 def _delayed_imports():
-    global type2container
     global _ListContainer
     global _Cube
     global _run
@@ -43,7 +42,7 @@ def _delayed_imports():
 
 class Container(object):
     """
-    CONTAINERS HOLD MULTIPLE FACTS AND CAN HANDLE
+    CONTAINERS HOLD MULTIPLE INDICES AND CAN HANDLE
     GENERAL JSON QUERY EXPRESSIONS ON ITS CONTENTS
     METADATA FOR A Container IS CALLED A Namespace
     """
@@ -115,10 +114,6 @@ class Container(object):
     def window(self, window):
         raise NotImplementedError()
 
-    def having(self, having):
-        _ = having
-        raise NotImplementedError()
-
     def format(self, format):
         _ = format
         raise NotImplementedError()
