@@ -5,27 +5,29 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this file,
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 #
-# Author: Kyle Lahnakoski (kyle@lahnakoski.com)
+# Contact: Kyle Lahnakoski (kyle@lahnakoski.com)
 #
 from __future__ import absolute_import, division, unicode_literals
 
-from collections import Mapping
 from copy import copy
+from importlib import import_module
 
 import jx_base
+import mo_math
 from jx_base.dimensions import Dimension
 from jx_base.domains import DefaultDomain, Domain, SetDomain
-from jx_base.expressions import Expression, FALSE, LeavesOp, QueryOp as QueryOp_, ScriptOp, TRUE, Variable, jx_expression
-from jx_base.utils import is_variable_name
+from jx_base.expressions import Expression, FALSE, LeavesOp, QueryOp as QueryOp_, ScriptOp, Variable, jx_expression
 from jx_base.language import is_expression, is_op
-from mo_dots import Data, FlatList, Null, coalesce, concat_field, is_container, is_data, is_list, listwrap, literal_field, relative_field, set_default, unwrap, unwraplist, wrap, is_many
-from mo_future import is_text, text_type
+from jx_base.utils import is_variable_name
+from mo_dots import Data, FlatList, Null, coalesce, concat_field, is_container, is_data, is_list, listwrap, \
+    literal_field, relative_field, set_default, unwrap, unwraplist, wrap, is_many
+from mo_future import is_text, text
 from mo_json import STRUCT
 from mo_json.typed_encoder import untype_path
 from mo_logs import Log
-import mo_math
 from mo_math import AND, UNION, is_number
 
+BAD_SELECT = "Expecting `value` or `aggregate` in select clause not {{select}}"
 DEFAULT_LIMIT = 10
 MAX_LIMIT = 10000
 DEFAULT_SELECT = Data(name="count", value=jx_expression("."), aggregate="count", default=0)
@@ -34,6 +36,7 @@ _jx = None
 _Column = None
 
 
+
 def _late_import():
     global _jx
     global _Column
@@ -46,15 +49,9 @@ def _late_import():
 
 
 class QueryOp(QueryOp_):
-    __slots__ = ["frum", "select", "edges", "groupby", "where", "window", "sort", "limit", "having", "format", "isLean"]
+    __slots__ = ["frum", "select", "edges", "groupby", "where", "window", "sort", "limit", "format", "chunk_size", "destination"]
 
-    # def __new__(cls, op=None, frum=None, select=None, edges=None, groupby=None, window=None, where=None, sort=None, limit=None, format=None):
-    #     output = object.__new__(cls)
-    #     for s in QueryOp.__slots__:
-    #         setattr(output, s, None)
-    #     return output
-
-    def __init__(self,frum, select=None, edges=None, groupby=None, window=None, where=None, sort=None, limit=None, format=None):
+    def __init__(self,frum, select=None, edges=None, groupby=None, window=None, where=None, sort=None, limit=None, format=None, chunk_size=None, destination=None):
         if isinstance(frum, jx_base.Table):
             pass
         else:
@@ -68,6 +65,8 @@ class QueryOp(QueryOp_):
         self.sort = sort
         self.limit = limit
         self.format = format
+        self.chunk_size = chunk_size
+        self.destination = destination
 
     def __data__(self):
         def select___data__():
@@ -210,10 +209,14 @@ class QueryOp(QueryOp_):
         output = QueryOp(
             frum=table,
             format=query.format,
-            limit=mo_math.min(MAX_LIMIT, coalesce(query.limit, DEFAULT_LIMIT))
+            chunk_size=query.chunk_size,
+            destination=query.destination,
         )
 
-        if query.select or isinstance(query.select, (Mapping, list)):
+        _import_temper_limit()
+        output.limit = temper_limit(query.limit, query)
+
+        if query.select or is_many(query.select) or is_data(query.select):
             output.select = _normalize_selects(query.select, query.frum, schema=schema)
         else:
             if query.edges or query.groupby:
@@ -235,13 +238,10 @@ class QueryOp(QueryOp_):
 
         output.where = _normalize_where({"and": listwrap(query.where)}, schema=schema)
         output.window = [_normalize_window(w) for w in listwrap(query.window)]
-        output.having = None
         output.sort = _normalize_sort(query.sort)
-        if not mo_math.is_integer(output.limit) or output.limit < 0:
+        if output.limit != None and (not mo_math.is_integer(output.limit) or output.limit < 0):
             Log.error("Expecting limit >= 0")
 
-        output.isLean = query.isLean
-
         return output
 
 
@@ -273,6 +273,18 @@ class QueryOp(QueryOp_):
         return output
 
 
+def temper_limit(limit, query):
+    return coalesce(query.limit, 10)
+
+
+def _import_temper_limit():
+    global temper_limit
+    try:
+        temper_limit = import_module("jx_elasticsearch.es52").temper_limit
+    except Exception as e:
+        pass
+
+
 canonical_aggregates = wrap({
     "cardinality": {"name":"cardinality", "default": 0},
     "count": {"name": "count", "default": 0},
@@ -285,7 +297,7 @@ canonical_aggregates = wrap({
 
 
 def _normalize_selects(selects, frum, schema=None, ):
-    if frum == None or isinstance(frum, (list, set, text_type)):
+    if frum == None or isinstance(frum, (list, set, text)):
         if is_list(selects):
             if len(selects) == 0:
                 return Null
@@ -329,7 +341,10 @@ def _normalize_select(select, frum, schema=None):
         return frum._normalize_select(canonical)
 
     output = []
-    if not select.value or select.value == ".":
+
+    if len(select) and not select.value:
+        Log.error(BAD_SELECT, select=select)
+    elif not select.value or select.value == ".":
         output.extend([
             set_default(
                 {
@@ -387,6 +402,8 @@ def _normalize_select_no_context(select, schema=None):
         output.name = coalesce(select.name, select.aggregate)
         if output.name:
             output.value = jx_expression(".", schema=schema)
+        elif len(select):
+            Log.error(BAD_SELECT, select=select)
         else:
             return Null
     elif is_text(select.value):
@@ -406,7 +423,7 @@ def _normalize_select_no_context(select, schema=None):
                 output.value = jx_expression(select.value, schema=schema)
     elif is_number(output.value):
         if not output.name:
-            output.name = text_type(output.value)
+            output.name = text(output.value)
         output.value = jx_expression(select.value, schema=schema)
     else:
         output.value = jx_expression(select.value, schema=schema)
@@ -552,7 +569,7 @@ def _normalize_group(edge, dim_index, limit, schema=None):
                     "put": {"name": literal_field(untype_path(prefix))},
                     "value": LeavesOp(Variable(prefix)),
                     "allowNulls": True,
-                    "dim":dim_index,
+                    "dim": dim_index,
                     "domain": {"type": "default"}
                 }])
 
@@ -565,7 +582,7 @@ def _normalize_group(edge, dim_index, limit, schema=None):
         }])
     else:
         edge = wrap(edge)
-        if (edge.domain and edge.domain.type != "default") or edge.allowNulls != None:
+        if (edge.domain and edge.domain.type != "default"):
             Log.error("groupby does not accept complicated domains")
 
         if not edge.name and not is_text(edge.value):
